<?php

namespace Drp\NativeEventApp\Test\Api\Auth;

use Drp\NativeEventApp\Auth\Facades\AuthInfoFinder;
use Drp\NativeEventApp\Auth\Facades\CodeGenerator;
use Drp\NativeEventApp\Contracts\MagicLinkSigner as MagicLinkSignerContract;
use Drp\NativeEventApp\Notifications\SendAuthCode;
use Drp\NativeEventApp\Test\Api\GraphQLTestCase;
use Drp\NativeEventApp\Test\Support\Fakes\Delegate;
use Drp\NativeEventApp\Test\Support\Fakes\MagicLinkSigner;
use Illuminate\Contracts\Mail\Mailer;
use Illuminate\Support\Facades\Notification;
use Nexmo\Client as NexmoClient;
use Spatie\Snapshots\MatchesSnapshots;

class AuthRequestTest extends GraphQLTestCase
{
    use MatchesSnapshots;

    /** @test */
    public function validates_signature(): void
    {
        $delegate = new Delegate;
        AuthInfoFinder::setAuthResolver(function ($authIdentifier) use ($delegate) {
            $this->assertEquals(1, $authIdentifier);

            return $delegate;
        });

        CodeGenerator::shouldReceive('authRequestCode')->andReturn('CODE');
        Notification::fake();

        $query = 'mutation {
        authRequest(authIdentifier:"1", expiry:"1234", signature: "1234", authMethod:SMS)
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));

        Notification::assertNothingSent();
    }

    /** @test */
    public function email_auth_token_to_auth_source(): void
    {
        $delegate = $this->prepareValidDelegate();
        CodeGenerator::shouldReceive('authRequestCode')->andReturn('CODE');
        Notification::fake();

        $query = 'mutation {
        authRequest(authIdentifier:"1", expiry:"1234", signature: "1234", authMethod:EMAIL)
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));

        Notification::assertSentTo(
            $delegate,
            SendAuthCode::class,
            function ($notification, $channels) {
                $this->assertEquals(['mail'], $channels);
                $this->assertEquals('CODE', $notification->tokenCode);
                return true;
            }
        );
    }

    /** @test */
    public function sms_auth_token_to_auth_source(): void
    {
        $delegate = $this->prepareValidDelegate();

        CodeGenerator::shouldReceive('authRequestCode')->andReturn('CODE');
        Notification::fake();

        $query = 'mutation {
        authRequest(authIdentifier:"1", expiry:"1234", signature: "1234", authMethod:SMS)
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));

        Notification::assertSentTo(
            $delegate,
            SendAuthCode::class,
            function ($notification, $channels) {
                $this->assertEquals(['nexmo'], $channels);
                $this->assertEquals('CODE', $notification->tokenCode);
                return true;
            }
        );
    }

    /** @test */
    public function handles_sms_sending_issues(): void
    {
        $this->mock(NexmoClient::class, static function ($mock): void {
            $mock
                ->shouldReceive('message')
                ->andThrow(new \Nexmo\Client\Exception\Exception('Original error', 400));
        });

        $this->prepareValidDelegate();
        CodeGenerator::shouldReceive('authRequestCode')->andReturn('CODE');

        $query = 'mutation {
        authRequest(authIdentifier:"1", expiry:"1234", signature: "1234", authMethod:SMS)
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    /** @test */
    public function handles_invalid_email(): void
    {
        $this->mock(Mailer::class, static function ($mock): void {
            $mock
                ->shouldReceive('send')
                ->andThrow(new \Swift_RfcComplianceException('Invalid email'));
        });

        $this->prepareValidDelegate();
        CodeGenerator::shouldReceive('authRequestCode')->andReturn('CODE');

        $query = 'mutation {
        authRequest(authIdentifier:"1", expiry:"1234", signature: "1234", authMethod:EMAIL)
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    /** @test */
    public function handles_email_sending_issues(): void
    {
        $this->mock(Mailer::class, static function ($mock): void {
            $mock
                ->shouldReceive('send')
                ->andThrow(new \Swift_SwiftException('Sending error'));
        });

        $this->prepareValidDelegate();
        CodeGenerator::shouldReceive('authRequestCode')->andReturn('CODE');

        $query = 'mutation {
        authRequest(authIdentifier:"1", expiry:"1234", signature: "1234", authMethod:EMAIL)
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    /** @test */
    public function handles_sending_issues(): void
    {
        $this->mock(Mailer::class, static function ($mock): void {
            $mock
                ->shouldReceive('send')
                ->andThrow(new \Exception('Some other error'));
        });

        $this->prepareValidDelegate();
        CodeGenerator::shouldReceive('authRequestCode')->andReturn('CODE');

        $query = 'mutation {
        authRequest(authIdentifier:"1", expiry:"1234", signature: "1234", authMethod:EMAIL)
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    /** @test */
    public function validates_sms_auth_method(): void
    {
        $delegate = $this->prepareValidDelegate();
        $delegate->phone = '11111111';
        CodeGenerator::shouldReceive('authRequestCode')->never();

        $query = 'mutation {
        authRequest(authIdentifier:"1", expiry:"1234", signature: "1234", authMethod:SMS)
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    /** @test */
    public function validates_email_auth_method(): void
    {
        $delegate = $this->prepareValidDelegate();
        $delegate->email = 'invalidemail.com';

        CodeGenerator::shouldReceive('authRequestCode')->never();

        $query = 'mutation {
        authRequest(authIdentifier:"1", expiry:"1234", signature: "1234", authMethod:EMAIL)
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    private function prepareValidDelegate(): Delegate
    {
        $delegate = new Delegate;
        $this->instance(MagicLinkSignerContract::class, new MagicLinkSigner);
        AuthInfoFinder::setAuthResolver(function ($authIdentifier) use ($delegate) {
            $this->assertEquals(1, $authIdentifier);

            return $delegate;
        });

        return $delegate;
    }
}
